# -*- coding: utf-8 -*-
import unittest
from tornado_ipware import get_client_ip


class Object(object):
    pass


class IPv4TestCase(unittest.TestCase):
    """IP address Test"""

    def test_meta_none(self):
        request = Object()
        request.headers = {}
        ip, routable = get_client_ip(request)
        self.assertIsNone(ip)
        self.assertFalse(routable)

    def test_meta_single(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_multi(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
            'REMOTE_ADDR': '177.139.233.133',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_multi_precedence_order(self):
        request = Object()
        request.headers = {
            'X_FORWARDED_FOR': '177.139.233.138, 198.84.193.157, 198.84.193.158',
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
            'REMOTE_ADDR': '177.139.233.133',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_proxy_order_left_most(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_order='left-most')
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_proxy_order_right_most(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_order='right-most')
        self.assertEqual(result, ("198.84.193.158", True))

    def test_meta_multi_precedence_private_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '10.0.0.0, 10.0.0.1, 10.0.0.2',
            'X_FORWARDED_FOR': '177.139.233.138, 198.84.193.157, 198.84.193.158',
            'REMOTE_ADDR': '177.139.233.133',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.138", True))

    def test_meta_multi_precedence_invalid_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': 'unknown, 10.0.0.1, 10.0.0.2',
            'X_FORWARDED_FOR': '177.139.233.138, 198.84.193.157, 198.84.193.158',
            'REMOTE_ADDR': '177.139.233.133',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.138", True))

    def test_meta_error_only(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': 'unknown, 177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request)
        self.assertEqual(result, (None, False))

    def test_meta_error_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': 'unknown, 177.139.233.139, 198.84.193.157, 198.84.193.158',
            'X_FORWARDED_FOR': '177.139.233.138, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.138", True))

    def test_meta_singleton(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_singleton_proxy_count(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139',
        }
        result = get_client_ip(request, proxy_count=1)
        self.assertEqual(result, (None, False))

    def test_meta_singleton_proxy_count_private(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '10.0.0.0',
            'HTTP_X_REAL_IP': '177.139.233.139',
        }
        result = get_client_ip(request, proxy_count=1)
        self.assertEqual(result, (None, False))

    def test_meta_singleton_private_fallback(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '10.0.0.0',
            'HTTP_X_REAL_IP': '177.139.233.139',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_proxy_trusted_ips(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_trusted_ips=['198.84.193.158'])
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_proxy_trusted_ips_proxy_count(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_count=2, proxy_trusted_ips=['198.84.193.158'])
        self.assertEqual(result, ("177.139.233.139", True))

    def test_meta_proxy_trusted_ips_proxy_count_less_error(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_count=2, proxy_trusted_ips=['198.84.193.158'])
        self.assertEqual(result, (None, False))

    def test_meta_proxy_trusted_ips_proxy_count_more_error(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        result = get_client_ip(request, proxy_count=1, proxy_trusted_ips=['198.84.193.158'])
        self.assertEqual(result, (None, False))

    def test_meta_proxy_trusted_ips_proxy_count_more_error_fallback(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
            'HTTP_X_REAL_IP': '177.139.233.139',
        }
        result = get_client_ip(request, proxy_count=1, proxy_trusted_ips=['198.84.193.158'])
        self.assertEqual(result, (None, False))

    def test_best_matched_ip(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '192.168.1.1',
            'REMOTE_ADDR': '177.31.233.133',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("177.31.233.133", True))

    def test_best_matched_ip_public(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '177.31.233.122',
            'REMOTE_ADDR': '177.31.233.133',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("177.31.233.122", True))

    def test_best_matched_ip_private(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '192.168.1.1',
            'REMOTE_ADDR': '127.0.0.1',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("192.168.1.1", False))

    def test_best_matched_ip_private_precedence(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '127.0.0.1',
            'REMOTE_ADDR': '192.168.1.1',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("192.168.1.1", False))

    def test_100_low_range_public(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '100.63.0.9',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("100.63.0.9", True))

    def test_100_block_private(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '100.76.0.9',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("100.76.0.9", False))

    def test_100_high_range_public(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '100.128.0.9',
        }
        ip = get_client_ip(request)
        self.assertEqual(ip, ("100.128.0.9", True))

    def test_request_header_order_specific(self):
        request = Object()
        request.headers = {
            'HTTP_X_REAL_IP': '192.168.1.1',
            'REMOTE_ADDR': '177.139.233.139',
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
        }
        ip = get_client_ip(request, request_header_order=['HTTP_X_FORWARDED_FOR'])
        self.assertEqual(ip, ("177.139.233.139", True))

    def test_request_header_order_multiple(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '177.139.233.139, 198.84.193.157, 198.84.193.158',
            'X_FORWARDED_FOR': '177.139.233.138, 198.84.193.157, 198.84.193.158',
            'REMOTE_ADDR': '177.139.233.133',
        }
        ip = get_client_ip(request, request_header_order=['X_FORWARDED_FOR', 'HTTP_X_FORWARDED_FOR'])
        self.assertEqual(ip, ("177.139.233.138", True))



# # -*- coding: utf-8 -*-
#
# from django.http import HttpRequest
# from django.test import TestCase
# from .. import utils as util
#
#
# class IPv4TestCase(TestCase):
#     """IP address Test"""
#
#
#     def test_is_valid_ip(self):
#         ip = '177.139.233.139'
#         self.assertTrue(util.is_valid_ip(ip))
#
#         ip = '3ffe:1900:4545:3:200:f8ff:fe21:67cf'
#         self.assertTrue(util.is_valid_ip(ip))
#
#     def test_is_invalid_ip(self):
#         ip = '177.139.233.139x'
#         self.assertFalse(util.is_valid_ip(ip))
#
#         ip = '3ffe:1900:4545:3:200:f8ff:fe21:67cz'
#         self.assertFalse(util.is_valid_ip(ip))
#
#     def test_is_private_ip(self):
#         ip = '127.0.0.1'
#         self.assertTrue(util.is_private_ip(ip))
#
#         ip = '::1/128'
#         self.assertTrue(util.is_private_ip(ip))
#
#     def test_is_public_ip(self):
#         ip = '177.139.233.139'
#         self.assertTrue(util.is_public_ip(ip))
#
#         ip = '74dc::02ba'
#         self.assertTrue(util.is_public_ip(ip))
#
#     def test_is_loopback_ip(self):
#         ip = '127.0.0.1'
#         self.assertTrue(util.is_loopback_ip(ip))
#
#         ip = '177.139.233.139'
#         self.assertFalse(util.is_loopback_ip(ip))
#
#         ip = '10.0.0.1'
#         self.assertFalse(util.is_loopback_ip(ip))
#
#         ip = '::1/128'
#         self.assertTrue(util.is_loopback_ip(ip))
#
#         ip = '74dc::02ba'
#         self.assertFalse(util.is_loopback_ip(ip))
#
#         ip = '2001:db8:'
#         self.assertFalse(util.is_loopback_ip(ip))
#
#     def test_http_request_meta_headers(self):
#         request = HttpRequest()
#         ip_str = '192.168.255.182, 10.0.0.0, 127.0.0.1, 198.84.193.157, 177.139.233.139,'
#         request.META = { 'HTTP_X_FORWARDED_FOR': ip_str }
#         value = util.get_request_meta(request, 'HTTP_X_FORWARDED_FOR')
#         self.assertEqual(value, ip_str)
#
#     def test_ips_from_strings(self):
#         ip_str = '192.168.255.182, 198.84.193.157, 177.139.233.139 ,'
#         result = util.get_ips_from_string(ip_str)
#         self.assertEqual(result, (['192.168.255.182', '198.84.193.157', '177.139.233.139'], 3))
#
#     def test_get_ip_info(self):
#         ip = '127.0.0.1'
#         result = util.get_ip_info(ip)
#         self.assertTrue(result, (ip, False))
#
#         ip = '10.0.01'
#         result = util.get_ip_info(ip)
#         self.assertTrue(result, (ip, False))
#
#         ip = '74dc::02ba'
#         result = util.get_ip_info(ip)
#         self.assertTrue(result, (ip, True))



import socket

from . import defaults as defs


def cleanup_ip(ip):
    """
    Given ip address string, it cleans it up
    """
    ip = ip.strip().lower()
    if (ip.startswith('::ffff:')):
        return ip.replace('::ffff:', '')
    return ip


def is_valid_ipv4(ip_str):
    """
    Check the validity of an IPv4 address
    """
    try:
        socket.inet_pton(socket.AF_INET, ip_str)
    except AttributeError:  # pragma: no cover
        try:  # Fall-back on legacy API or False
            socket.inet_aton(ip_str)
        except (AttributeError, socket.error):
            return False
        return ip_str.count('.') == 3
    except socket.error:
        return False
    return True


def is_valid_ipv6(ip_str):
    """
    Check the validity of an IPv6 address
    """
    try:
        socket.inet_pton(socket.AF_INET6, ip_str)
    except socket.error:
        return False
    return True


def is_valid_ip(ip_str):
    """
    Check the validity of an IP address
    """
    return is_valid_ipv4(ip_str) or is_valid_ipv6(ip_str)


def is_private_ip(ip_str):
    """
    Returns true of ip_str is private & not routable, else return false
    """
    return ip_str.startswith(defs.IPWARE_NON_PUBLIC_IP_PREFIX)


def is_public_ip(ip_str):
    """
    Returns true of ip_str is public & routable, else return false
    """
    return not is_private_ip(ip_str)


def is_loopback_ip(ip_str):
    """
    Returns true of ip_str is public & routable, else return false
    """
    return ip_str.startswith(defs.IPWARE_LOOPBACK_PREFIX)


def get_request_header(request, key):
    """
    Given a key, it returns a cleaned up version of the value from request.META, or None
    """
    value = request.headers.get(key, request.headers.get(key.replace('_', '-'), '')).strip()
    if value == '':
        return None
    return value


def get_ips_from_string(ip_str):
    """
    Given a string, it returns a list of one or more valid IP addresses
    """
    ip_list = []

    for ip in ip_str.split(','):
        clean_ip = ip.strip().lower()
        if clean_ip:
            ip_list.append(clean_ip)

    ip_count = len(ip_list)
    if ip_count > 0:
        if is_valid_ip(ip_list[0]) and is_valid_ip(ip_list[-1]):
            return ip_list, ip_count

    return [], 0


def get_ip_info(ip_str):
    """
    Given a string, it returns a tuple of (IP, Routable).
    """
    ip = None
    is_routable_ip = False
    clean_ip = cleanup_ip(ip_str)
    if is_valid_ip(clean_ip):
        ip = clean_ip
        is_routable_ip = is_public_ip(ip)
    return ip, is_routable_ip


def get_best_ip(last_ip, next_ip):
    """
    Given two IP addresses, it returns the the best match ip.
    Order of precedence is (Public, Private, Loopback, None)
    Right-most IP is returned
    """
    if last_ip is None:
        return next_ip
    if is_public_ip(last_ip) and not is_public_ip(next_ip):
        return last_ip
    if is_private_ip(last_ip) and is_loopback_ip(next_ip):
        return last_ip
    return next_ip



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


name = 'tornado-ipware'
package = 'tornado_ipware'
description = "A Tornado utility application that returns client's real IP address"
url = 'https://github.com/lumbrefrio/tornado-ipware'
author = 'Bill Batey'
author_email = 'lumbrefrio@gmail.com'
license = 'MIT'
install_requires = ['']
classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Topic :: Utilities'
]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s' && git push --tags" % args)
    sys.exit()


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires,
    classifiers=classifiers
)



# -*- coding: utf-8 -*-
import unittest
from tornado_ipware import get_client_ip


class Object(object):
    pass


class IPv4TestCase(unittest.TestCase):
    """IP address Test"""

    def test_meta_none(self):
        request = Object()
        request.headers = {}
        ip, routable = get_client_ip(request)
        self.assertIsNone(ip)
        self.assertFalse(routable)

    def test_meta_single(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_multi(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'REMOTE_ADDR': '74dc::02bc',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_multi_precedence_order(self):
        request = Object()
        request.headers = {
            'X_FORWARDED_FOR': '74dc::02be, 74dc::02bf',
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'REMOTE_ADDR': '74dc::02bc',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_proxy_order_left_most(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_order='left-most')
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_proxy_order_right_most(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_order='right-most')
        self.assertEqual(result, ("74dc::02bb", True))

    def test_meta_multi_precedence_private_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '2001:db8:, ::1',
            'X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'REMOTE_ADDR': '74dc::02bc',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_multi_precedence_invalid_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': 'unknown, 2001:db8:, ::1',
            'X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'REMOTE_ADDR': '74dc::02bc',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_error_only(self):
        request = Object()
        request.headers = {
            'X_FORWARDED_FOR': 'unknown, 3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request)
        self.assertEqual(result, (None, False))

    def test_meta_error_first(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': 'unknown, 3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_singleton(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_singleton_proxy_count(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf',
            'HTTP_X_REAL_IP': '74dc::02ba',
        }
        result = get_client_ip(request, proxy_count=1)
        self.assertEqual(result, (None, False))

    def test_meta_singleton_proxy_count_private(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '::1',
            'HTTP_X_REAL_IP': '3ffe:1900:4545:3:200:f8ff:fe21:67cf',
        }
        result = get_client_ip(request, proxy_count=1)
        self.assertEqual(result, (None, False))

    def test_meta_singleton_private_fallback(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '::1',
            'HTTP_X_REAL_IP': '3ffe:1900:4545:3:200:f8ff:fe21:67cf',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_proxy_trusted_ips(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_trusted_ips=['74dc::02bb'])
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_proxy_trusted_ips_proxy_count(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_count=2, proxy_trusted_ips=['74dc::02bb'])
        self.assertEqual(result, ("3ffe:1900:4545:3:200:f8ff:fe21:67cf", True))

    def test_meta_proxy_trusted_ips_proxy_count_less_error(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_count=2, proxy_trusted_ips=['74dc::02bb'])
        self.assertEqual(result, (None, False))

    def test_meta_proxy_trusted_ips_proxy_count_more_error(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
        }
        result = get_client_ip(request, proxy_count=1, proxy_trusted_ips=['74dc::02bb'])
        self.assertEqual(result, (None, False))

    def test_meta_proxy_trusted_ips_proxy_count_more_error_ignore_fallback(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '3ffe:1900:4545:3:200:f8ff:fe21:67cf, 74dc::02ba, 74dc::02bb',
            'HTTP_X_REAL_IP': '74dc::02bb',
        }
        result = get_client_ip(request, proxy_count=1, proxy_trusted_ips=['74dc::02bb'])
        self.assertEqual(result, (None, False))


class IPv6EncapsulationOfIPv4TestCase(unittest.TestCase):
    """IPv6 Encapsulation of IPv4 - IP address Test"""

    def test_ipv6_encapsulation_of_ipv4_private(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '::ffff:127.0.0.1',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ('127.0.0.1', False))

    def test_ipv6_encapsulation_of_ipv4_public(self):
        request = Object()
        request.headers = {
            'HTTP_X_FORWARDED_FOR': '::ffff:177.139.233.139',
        }
        result = get_client_ip(request)
        self.assertEqual(result, ('177.139.233.139', True))
